#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
#
# This script is licensed under GNU GPL version 2.0 or above
# (c) 2024 Antonio J. Delgado
"""Nextcloud Password client"""

import json
import sys
import os
import logging
from logging.handlers import SysLogHandler
import click
import click_config_file
from nextcloud import NextcloudHandler


class NcPasswordClient:
    '''Nextcloud Password Client'''

    def __init__(self, debug_level, log_file, host, user, api_token, cse_password):
        self.config = {}
        self.config['debug_level'] = debug_level
        if log_file is None:
            self.config['log_file'] = os.path.join(
                os.environ.get(
                    'HOME',
                    os.environ.get(
                        'USERPROFILE',
                        os.getcwd()
                    )
                ),
                'log',
                'nc_password_client.log'
            )
        self._init_log()
        params = {
            "host": host,
            "user": user,
            "api_token": api_token,
            "cse_password": cse_password
        }
        self.nc = NextcloudHandler(params)

    def get_password(self, name):
        '''Get a single password'''
        print(json.dumps(self.nc.get_password(name), indent=2))

    def list_passwords(self):
        '''List all passwords'''    
        print(json.dumps(self.nc.list_passwords(), indent=2))

    def _init_log(self):
        ''' Initialize log object '''
        self._log = logging.getLogger("nc_password_client")
        self._log.setLevel(logging.DEBUG)

        sysloghandler = SysLogHandler()
        sysloghandler.setLevel(logging.DEBUG)
        self._log.addHandler(sysloghandler)

        streamhandler = logging.StreamHandler(sys.stdout)
        streamhandler.setLevel(
            logging.getLevelName(self.config.get("debug_level", 'INFO'))
        )
        self._log.addHandler(streamhandler)

        if 'log_file' in self.config:
            log_file = self.config['log_file']
        else:
            home_folder = os.environ.get(
                'HOME', os.environ.get('USERPROFILE', '')
            )
            log_folder = os.path.join(home_folder, "log")
            log_file = os.path.join(log_folder, "nc_password_client.log")

        if not os.path.exists(os.path.dirname(log_file)):
            os.mkdir(os.path.dirname(log_file))

        filehandler = logging.handlers.RotatingFileHandler(
            log_file, maxBytes=102400000
        )
        # create formatter
        formatter = logging.Formatter(
            '%(asctime)s %(name)-12s %(levelname)-8s %(message)s'
        )
        filehandler.setFormatter(formatter)
        filehandler.setLevel(logging.DEBUG)
        self._log.addHandler(filehandler)
        return True


@click.group()
@click.option(
    "--debug-level",
    "-d",
    default="INFO",
    type=click.Choice(
        ["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG", "NOTSET"],
        case_sensitive=False,
    ),
    help='Set the debug level for the standard output.'
)
@click.option('--log-file', '-l', help="File to store all debug messages.")
# @click.option("--dummy","-n", is_flag=True,
# help="Don't do anything, just show what would be done.")
@click.option(
    '--host', '-H',
    required=True,
    help='Nextcloud host name'
)
@click.option(
    '--user', '-u',
    required=True,
    help='Nextcloud user name'
)
@click.option(
    '--api-token', '-a',
    required=True,
    help='Nextcloud user\'s application token'
)
@click.option(
    '--cse-password', '-p',
    help='Nextcloud user\'s end-to-end encryption password'
)
@click_config_file.configuration_option()
@click.pass_context
def cli(ctx, debug_level, log_file, host, user, api_token, cse_password):
    '''Client function to pass context'''
    ctx.ensure_object(dict)
    ctx.obj['NcPasswordClient'] = NcPasswordClient(debug_level, log_file, host, user, api_token, cse_password)

@cli.command()
@click.option('--name', '-n', required=True, help='Name of the password to show')
@click_config_file.configuration_option()
@click.pass_context
def show(ctx, name):
    '''Show a single password'''
    ctx.obj['NcPasswordClient'].get_password(name)

@cli.command()
@click_config_file.configuration_option()
@click.pass_context
def list(ctx):
    '''Show all password'''
    ctx.obj['NcPasswordClient'].list_passwords()

if __name__ == "__main__":
    cli(obj={})
